@@ -19,11 +19,11 @@ Description:
 
 #pragma hdrstop
 // any other headers (not precompiled)
-#include "GrClient.h"
-#include "ITextSource.h"
-#include "IGrJustifier.h"
-#include "GraphiteProcess.h"
-#include "GrDebug.h"
+#include <graphite/GrClient.h>
+#include <graphite/ITextSource.h>
+#include <graphite/IGrJustifier.h>
+#include <graphite/GraphiteProcess.h>
+#include <graphite/GrDebug.h>
 
 #include "GrJustifier.h"
 
@@ -189,7 +189,7 @@ LMainLoop:
 			cUnits = 0;
 			cStretchable = 0;
 	
-			for (iiGlyph = 0; iiGlyph < viGlyphs.size(); iiGlyph++)
+			for (iiGlyph = 0; iiGlyph < signed(viGlyphs.size()); iiGlyph++)
 			{
 				if (vdxStep[iiGlyph] > 0 && fIgnoreStepGlyphs)
 					continue; // leave step-glyphs as they are
@@ -231,7 +231,7 @@ LMainLoop:
 			int cNonStepUnits = 0;
 			int cStretchableNonStep = 0;
 			bool fReloop = false;
-			for (iiGlyph = 0; iiGlyph < viGlyphs.size(); iiGlyph++)
+			for (iiGlyph = 0; iiGlyph < signed(viGlyphs.size()); iiGlyph++)
 			{
 				if (vdxStep[iiGlyph] > 1)
 				{
@@ -307,13 +307,13 @@ LMainLoop:
 		int dxRemainder = dxStretchNeeded - dxStretchAchieved;
 		if (0 < dxRemainder && dxRemainder < cStretchable)
 		{
-			if (cStretchable < viGlyphs.size() || fStep)
+			if (cStretchable < signed(viGlyphs.size()) || fStep)
 			{
 				//	Make sub-lists using the glyphs that are still stretchable.
 				std::vector<int> vdxStretchRem;
 				std::vector<int> vdxWidthRem;
 				std::vector<int> viiGlyphsRem;
-				for (iiGlyph = 0; iiGlyph < viGlyphs.size(); iiGlyph++)
+				for (iiGlyph = 0; iiGlyph < signed(viGlyphs.size()); iiGlyph++)
 				{
 					if (vdxStretchLeft[iiGlyph] > 0 && vdxStep[iiGlyph] == 0)
 					{
@@ -322,7 +322,8 @@ LMainLoop:
 						vdxWidthRem.push_back(vdxWidth[iiGlyph]);
 					}
 				}
-				Assert(viiGlyphsRem.size() == cStretchable);
+				Assert(viiGlyphsRem.size() == size_t(cStretchable));
+				// Distribute remaining stretch evenly throughout the stretchable glyphs.
 				DistributeRemainder(vdxWidthRem, vdxStretchRem, dxRemainder, 0, vdxWidthRem.size(),
 					&dxStretchAchieved);
 				for (int iiiGlyph = 0; iiiGlyph < cStretchable; iiiGlyph++)
@@ -339,37 +340,46 @@ LMainLoop:
 					&dxStretchAchieved);
 			}
 		}
-		// otherwise we assume left-over is cannot be handled
+		// otherwise we assume left-over cannot be handled
 	
 		// Assign the widths to the glyphs.
 	
-		for (iiGlyph = 0; iiGlyph < viGlyphs.size(); iiGlyph++)
+		for (iiGlyph = 0; iiGlyph < signed(viGlyphs.size()); iiGlyph++)
 		{
 			int dxThis = vdxWidth[iiGlyph] * ((fShrinking) ? -1 : 1);
 			if (vdxStep[iiGlyph] == 0)
+			{
+				// Non-step (fine-grained) glyph:
 				pgje->setGlyphAttribute(viGlyphs[iiGlyph], gr::kjgatWidth, 1, dxThis);
+			}
 			else
 			{
-				// Set the actual number of steps allowed. This is more accurate than
-				// setting the pixels and then converting to font em-units.
+				// Step-glyph: set the actual number of steps allowed. This is more accurate
+				// than setting the pixels and then converting to font em-units.
 				Assert(int(dxThis) % vdxStep[iiGlyph] == 0); // width divides evenly into steps
 				int cSteps = dxThis/vdxStep[iiGlyph];
 				pgje->setGlyphAttribute(viGlyphs[iiGlyph], gr::kjgatWidthInSteps, 1, cSteps);
 			}
 		}
 	}
-LLeave:
+//LLeave:
 
 	if (dxStretchAchieved == dxStretchNeeded)
 		return gr::kresOk;
 	else
 	{
 #ifdef WIN32
-		std::string strTmp("justification failed by ");
-		strTmp += " units (width needed = ";
-		strTmp += dxDesiredWidth;
-        strTmp += ")\n";
-		OutputDebugString(strTmp.c_str());
+		wchar_t rgchw[20];
+		std::fill_n(rgchw, 20, 0);
+		_itow(dxStretchNeeded - dxStretchAchieved, rgchw, 10);
+		std::wstring strTmp(L"justification failed by ");
+		strTmp += rgchw;
+		strTmp += L" units (width needed = ";
+		std::fill_n(rgchw, 10, 0);
+		_itow(dxDesiredWidth, rgchw, 10);
+		strTmp += rgchw;
+		strTmp += L")\n";
+//		OutputDebugString(strTmp.c_str());
 #else
 		Assert(fprintf(stderr,
 			   "justification failed by %d units (width needed = %d)\n",
@@ -392,6 +402,7 @@ void GrJustifier::DistributeRemainder(std::vector<int> & vdxWidths, std::vector<
 	Assert(dx <= iiLim - iiMin);
 	if (iiMin + 1 == iiLim)
 	{
+		// One glyph to apply the stretch to.
 		int dxThis = min(dx, vdxStretch[iiMin]);
 		Assert(dxThis == 1); // we're never adjusting by more than 1, and the glyph should be
 							 // adjustable by that much
@@ -433,7 +444,7 @@ int GrJustifier::Lcm(std::vector<int> & vnWeights, std::vector<int> & vnMFactors
 	std::vector<int> vnPowersPerPrime;
 	vnPowersPerPrime.resize(g_cnPrimes);
 	int nWtMax = 1;
-	for (int inWt = 0; inWt < vnWeights.size(); inWt++)
+	for (int inWt = 0; inWt < signed(vnWeights.size()); inWt++)
 	{
 		int inMax = PrimeFactors(vnWeights[inWt], vnPowersPerPrime);
 		for (inPrime = 0; inPrime <= inMax; inPrime++)
